//Copyright 2013 Michael Leitner, distributed as free softtware,
//see COPYING for more details

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>

//link with -lX11
//reads a ppm image on STDIN, displays it in a window, writes the coordinates of mouse-clicks into this window to OUTFILE,
// and exits on pressed key. Note: for passing arbitrary images to it use convert FILEIN ppm:- | pixpick OUTFILE

void malformed() {
	printf("malformed header -- only binary PPM supported!\n");
	exit(EXIT_FAILURE);
}

void consume_white_comment() {
	char i;
	while (1) {
		scanf(" %c",&i);
		if (i=='#') scanf("%*[^\n]\n");
		else break;
	}
	ungetc(i,stdin);
}

int main(int argc, char**argv) {
	int i,width,height;
	FILE * fid;
	
	if ((argc!=2)||(strcmp(argv[1],"-h")==0)) {
		printf("Usage...\n");
		exit(EXIT_FAILURE);
	}
	fid=fopen(argv[1],"w");
	if (fid==NULL) {
		printf("could not open %s\n",argv[1]);
		exit(EXIT_FAILURE);
	}
	fclose(fid);
	
	consume_white_comment();
	if (getchar()!='P') malformed();
	if (getchar()!='6') malformed();
	consume_white_comment();
	if (scanf("%d",&width)!=1) malformed();
	consume_white_comment();
	if (scanf("%d",&height)!=1) malformed();
	consume_white_comment();
	if (scanf("%d",&i)!=1) malformed();
	scanf("%*c");
	if (i!=255) {
		printf("only 8-bit colours supported!\n");
		exit(EXIT_FAILURE);
	}


	unsigned char * xdata=malloc(width*height*4);
	if (fread(xdata,3,width*height,stdin)!=width*height) {
		printf("inconsistent header/data section\n");
		exit(EXIT_FAILURE);
	}
	for (i=width*height-1;i>=0;i--) {
		xdata[i*4]=xdata[i*3+2];
		xdata[i*4+1]=xdata[i*3+1];
		xdata[i*4+2]=xdata[i*3];
	}

	Display *display=XOpenDisplay(NULL);
	XImage *img=XCreateImage(display,CopyFromParent,24,ZPixmap,0,xdata,width,height,32,0);
	XInitImage(img);

	Window window=XCreateSimpleWindow(display,RootWindow(display,0),0,0,width,height,1,BlackPixel(display,0),WhitePixel(display,0));
	
	XSelectInput(display,window,ButtonPressMask|ExposureMask|KeyPressMask);
	
	XPutImage(display, window, DefaultGC(display, 0), img, 0, 0, 0, 0, width, height);
	XMapWindow(display,window);

	XEvent event;
	while (1) {
		XNextEvent(display,&event);
		
		switch(event.type)
		{
		case Expose:
		    XPutImage(display, window, DefaultGC(display, 0), img, 0, 0, 0, 0, width, height);
		    break;
		case KeyPress:
		    exit(0);
		case ButtonPress:
			fid=fopen(argv[1],"a");
		    fprintf(fid,"%d %d\n",event.xbutton.x,event.xbutton.y);
		    fclose(fid);
		}
	}

	return 0;
}
